--- app_queue.c.orig	2008-11-24 10:27:58.000000000 -0500
+++ app_queue.c	2008-11-24 12:28:27.000000000 -0500
@@ -405,6 +405,7 @@
 	int periodicannouncefrequency;      /*!< How often to play periodic announcement */
 	int roundingseconds;                /*!< How many seconds do we round to? */
 	int holdtime;                       /*!< Current avg holdtime, based on an exponential average */
+	int talktime;			    /*!< Current avg talktime, based on the same exponential average */
 	int callscompleted;                 /*!< Number of queue calls completed */
 	int callsabandoned;                 /*!< Number of queue calls abandoned */
 	int servicelevel;                   /*!< seconds setting for servicelevel*/
@@ -863,6 +864,7 @@
 static void clear_queue(struct call_queue *q)
 {
 	q->holdtime = 0;
+	q->talktime = 0;
 	q->callscompleted = 0;
 	q->callsabandoned = 0;
 	q->callscompletedinsl = 0;
@@ -2500,14 +2502,19 @@
 	return res;
 }
 
-static int update_queue(struct call_queue *q, struct member *member, int callcompletedinsl)
+static int update_queue(struct call_queue *q, struct member *member, int callcompletedinsl, int newtalktime)
 {
+	int oldtalktime;
+	
 	ast_mutex_lock(&q->lock);
 	time(&member->lastcall);
 	member->calls++;
 	q->callscompleted++;
 	if (callcompletedinsl)
 		q->callscompletedinsl++;
+        /* Calculate talktime using the same exponential average as holdtime code*/
+        oldtalktime = q->talktime;
+        q->talktime = (((oldtalktime << 2) - oldtalktime) + newtalktime) >> 2;
 	ast_mutex_unlock(&q->lock);
 	return 0;
 }
@@ -3201,7 +3208,7 @@
 				ast_channel_datastore_free(transfer_ds);
 			}
 			ast_channel_unlock(qe->chan);
-			update_queue(qe->parent, member, callcompletedinsl);
+			update_queue(qe->parent, member, callcompletedinsl, (time(NULL) - callstart));
 		}
 
 		if (bridge != AST_PBX_NO_HANGUP_PEER && bridge != AST_PBX_NO_HANGUP_PEER_PARKED)
@@ -4526,12 +4533,12 @@
 		if (q->callscompleted > 0)
 			sl = 100 * ((float) q->callscompletedinsl / (float) q->callscompleted);
 		if (s)
-			astman_append(s, "%-12.12s has %d calls (max %s) in '%s' strategy (%ds holdtime), W:%d, C:%d, A:%d, SL:%2.1f%% within %ds%s",
-				q->name, q->count, max_buf, int2strat(q->strategy), q->holdtime, q->weight,
+			astman_append(s, "%-12.12s has %d calls (max %s) in '%s' strategy (%ds holdtime, %ds talktime), W:%d, C:%d, A:%d, SL:%2.1f%% within %ds%s",
+				q->name, q->count, max_buf, int2strat(q->strategy), q->holdtime, q->talktime, q->weight,
 				q->callscompleted, q->callsabandoned,sl,q->servicelevel, term);
 		else
-			ast_cli(fd, "%-12.12s has %d calls (max %s) in '%s' strategy (%ds holdtime), W:%d, C:%d, A:%d, SL:%2.1f%% within %ds%s",
-				q->name, q->count, max_buf, int2strat(q->strategy), q->holdtime, q->weight, q->callscompleted, q->callsabandoned,sl,q->servicelevel, term);
+			ast_cli(fd, "%-12.12s has %d calls (max %s) in '%s' strategy (%ds holdtime, %ds talktime), W:%d, C:%d, A:%d, SL:%2.1f%% within %ds%s",
+				q->name, q->count, max_buf, int2strat(q->strategy), q->holdtime, q->talktime, q->weight, q->callscompleted, q->callsabandoned,sl,q->servicelevel, term);
 		if (ao2_container_count(q->members)) {
 			if (s)
 				astman_append(s, "   Members: %s", term);
@@ -4677,6 +4684,7 @@
 				"Max: %d\r\n"
 				"Calls: %d\r\n"
 				"Holdtime: %d\r\n"
+				"Talktime: %d\r\n"
 				"Completed: %d\r\n"
 				"Abandoned: %d\r\n"
 				"ServiceLevel: %d\r\n"
@@ -4684,7 +4692,7 @@
 				"Weight: %d\r\n"
 				"%s"
 				"\r\n",
-				q->name, q->maxlen, q->count, q->holdtime, q->callscompleted,
+				q->name, q->maxlen, q->count, q->holdtime, q->talktime, q->callscompleted,
 				q->callsabandoned, q->servicelevel, sl, q->weight, idText);
 			/* List Queue Members */
 			mem_iter = ao2_iterator_init(q->members, 0);
